package hracilovacastka;

import java.util.Random;

/**
 *
 * @author Dana Nejedlová na základě projektu pana Vladimíra Caldy
 */
public enum Hrac {

    CLOVEK("člověk") {

                @Override
                public int tah(int soucet, int cilovaCastka, int minimumPridavani, int maximumPridavani) {
                    int castka;
                    int zbyva = cilovaCastka - soucet;
                    if (zbyva > maximumPridavani) {
                        castka = Vstup.zadejCeleCisloVIntervalu(minimumPridavani, maximumPridavani);
                    } else if (zbyva < minimumPridavani) {
                        castka = Vstup.zadejCeleCisloVIntervalu(zbyva, maximumPridavani);
                    } else {
                        castka = Vstup.zadejCeleCisloVIntervalu(minimumPridavani, zbyva);
                    }
                    return (castka);
                }

                @Override
                public Hrac hrajePoMne() {
                    return POCITAC;
                }

                @Override
                public Hrac hrajePredeMnou() {
                    return POCITAC;
                }
            },
    POCITAC("počítač") {
                /*
                 *
                 * Algorithm of the generalized version of the
                 * https://en.wikipedia.org/wiki/Nim#The_100_game
                 * Each configuration has its winning sequence of values such
                 * that if any of them is reached by the particular player,
                 * the next value in the sequence can also be reached by this player.
                 * The sequence is the following:
                 * a      = cilovaCastka % b
                 * a + 1b
                 * a + 2b
                 * a + 3b
                 * ...
                 * a + nb = cilovaCastka
                 * n = cilovaCastka / b at the end of the game
                 * n = soucet / b in the middle of the game
                 * b = minimumPridavani + maximumPridavani
                 * b = difference of two subsequent values is such that at least two moves
                 * must be played to get from one value in the sequence to the subsequent value.
                 * In this way the player who has reached one of those values can reach the next
                 * value in the sequence whatever the move of his opponent is.
                 * 
                 */
                @Override
                public int tah(int soucet, int cilovaCastka, int minimumPridavani, int maximumPridavani) {
                    int a, b, n, pridat, castka;
                    b = minimumPridavani + maximumPridavani;
                    a = cilovaCastka % b;
                    n = 0;
                    while (a + n * b < soucet) {
                        n++;
                    }
                    pridat = a + n * b - soucet;
                    /*System.out.println("a = " + a);
                    System.out.println("b = " + b);
                    System.out.println("n = " + n);
                    System.out.println("pridat = " + pridat);*/
                    if (pridat < minimumPridavani || pridat > maximumPridavani) {
                        castka = minimumPridavani + rand.nextInt(1 + maximumPridavani - minimumPridavani);
                        System.out.println("Můj tah je (random)  " + castka);
                    } else {
                        castka = pridat;
                        System.out.println("Můj tah je (winning) " + castka);
                    }
                    return (castka);
                }

                @Override
                public Hrac hrajePoMne() {
                    return CLOVEK;
                }

                @Override
                public Hrac hrajePredeMnou() {
                    return CLOVEK;
                }
            };

    static Random rand = new Random();

    public abstract int tah(int soucet, int cilovaCastka, int minimumPridavani, int maximumPridavani);

    public abstract Hrac hrajePoMne();

    public abstract Hrac hrajePredeMnou(); // Potřebné, když by se střídali více než dva hráči.

    private final String kdoJsem;

    public void predstaveniHrace() {
        System.out.println("\nNa tahu je " + this.kdoJsem + ".");
    }

    public void ohlaseniViteze() {
        System.out.println("Vyhrál " + this.kdoJsem + ".");
    }

    Hrac(String kdoJsem) {
        this.kdoJsem = kdoJsem;
    }
}
